# -*- coding: utf-8 -*-
# The Keccak sponge function, designed by Guido Bertoni, Joan Daemen,
# Michaël Peeters and Gilles Van Assche. For more information, feedback or
# questions, please refer to our website: http://keccak.noekeon.org/
#
# Implementation by Renaud Bauvin,
# hereby denoted as "the implementer".
#
# To the extent possible under law, the implementer has waived all copyright
# and related or neighboring rights to the source code in this file.
# http://creativecommons.org/publicdomain/zero/1.0/
#
# Source: http://keccak.noekeon.org/files.html | Version: 3.2

import math

class KeccakError(Exception):
    """Class of error used in the Keccak implementation

    Use: raise KeccakError.KeccakError("Text to be displayed")"""

    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)


class Keccak:
    """
    Class implementing the Keccak sponge function
    """
    def __init__(self, b=1600):
        """Constructor:

        b: parameter b, must be 25, 50, 100, 200, 400, 800 or 1600 (default value)"""
        self.setB(b)

    def setB(self,b):
        """Set the value of the parameter b (and thus w,l and nr)

        b: parameter b, must be choosen among [25, 50, 100, 200, 400, 800, 1600]
        """

        if b not in [25, 50, 100, 200, 400, 800, 1600]:
            raise KeccakError.KeccakError('b value not supported - use 25, 50, 100, 200, 400, 800 or 1600')

        # Update all the parameters based on the used value of b
        self.b=b
        self.w=b//25
        self.l=int(math.log(self.w,2))
        self.nr=12+2*self.l

    # Constants

    ## Round constants
    RC=[0x0000000000000001,
        0x0000000000008082,
        0x800000000000808A,
        0x8000000080008000,
        0x000000000000808B,
        0x0000000080000001,
        0x8000000080008081,
        0x8000000000008009,
        0x000000000000008A,
        0x0000000000000088,
        0x0000000080008009,
        0x000000008000000A,
        0x000000008000808B,
        0x800000000000008B,
        0x8000000000008089,
        0x8000000000008003,
        0x8000000000008002,
        0x8000000000000080,
        0x000000000000800A,
        0x800000008000000A,
        0x8000000080008081,
        0x8000000000008080,
        0x0000000080000001,
        0x8000000080008008]

    ## Rotation offsets
    r=[[0,    36,     3,    41,    18]    ,
       [1,    44,    10,    45,     2]    ,
       [62,    6,    43,    15,    61]    ,
       [28,   55,    25,    21,    56]    ,
       [27,   20,    39,     8,    14]    ]

    ## Generic utility functions

    def rot(self,x,n):
        """Bitwise rotation (to the left) of n bits considering the \
        string of bits is w bits long"""

        n = n%self.w
        return ((x>>(self.w-n))+(x<<n))%(1<<self.w)

    def fromHexStringToLane(self, string):
        """Convert a string of bytes written in hexadecimal to a lane value"""

        #Check that the string has an even number of characters i.e. whole number of bytes
        if len(string)%2!=0:
            raise KeccakError.KeccakError("The provided string does not end with a full byte")

        #Perform the modification
        temp=''
        nrBytes=len(string)//2
        for i in range(nrBytes):
            offset=(nrBytes-i-1)*2
            temp+=string[offset:offset+2]
        return int(temp, 16)

    def fromLaneToHexString(self, lane):
        """Convert a lane value to a string of bytes written in hexadecimal"""

        laneHexBE = (("%%0%dX" % (self.w//4)) % lane)
        #Perform the modification
        temp=''
        nrBytes=len(laneHexBE)//2
        for i in range(nrBytes):
            offset=(nrBytes-i-1)*2
            temp+=laneHexBE[offset:offset+2]
        return temp.upper()

    def printState(self, state, info):
        """Print on screen the state of the sponge function preceded by \
        string info

        state: state of the sponge function
        info: a string of characters used as identifier"""

        print("Current value of state: %s" % (info))
        for y in range(5):
            line=[]
            for x in range(5):
                 line.append(hex(state[x][y]))
            print('\t%s' % line)

    ### Conversion functions String <-> Table (and vice-versa)

    def convertStrToTable(self,string):
        """Convert a string of bytes to its 5×5 matrix representation

        string: string of bytes of hex-coded bytes (e.g. '9A2C...')"""

        #Check that input paramaters
        if self.w%8!= 0:
            raise KeccakError("w is not a multiple of 8")
        if len(string)!=2*(self.b)//8:
            raise KeccakError.KeccakError("string can't be divided in 25 blocks of w bits\
            i.e. string must have exactly b bits")

        #Convert
        output=[[0,0,0,0,0],
                [0,0,0,0,0],
                [0,0,0,0,0],
                [0,0,0,0,0],
                [0,0,0,0,0]]
        for x in range(5):
            for y in range(5):
                offset=2*((5*y+x)*self.w)//8
                output[x][y]=self.fromHexStringToLane(string[offset:offset+(2*self.w//8)])
        return output

    def convertTableToStr(self,table):
        """Convert a 5×5 matrix representation to its string representation"""

        #Check input format
        if self.w%8!= 0:
            raise KeccakError.KeccakError("w is not a multiple of 8")
        if (len(table)!=5) or (False in [len(row)==5 for row in table]):
            raise KeccakError.KeccakError("table must be 5×5")

        #Convert
        output=['']*25
        for x in range(5):
            for y in range(5):
                output[5*y+x]=self.fromLaneToHexString(table[x][y])
        output =''.join(output).upper()
        return output

    def Round(self,A,RCfixed):
        """Perform one round of computation as defined in the Keccak-f permutation

        A: current state (5×5 matrix)
        RCfixed: value of round constant to use (integer)
        """

        #Initialisation of temporary variables
        B=[[0,0,0,0,0],
           [0,0,0,0,0],
           [0,0,0,0,0],
           [0,0,0,0,0],
           [0,0,0,0,0]]
        C= [0,0,0,0,0]
        D= [0,0,0,0,0]

        #Theta step
        for x in range(5):
            C[x] = A[x][0]^A[x][1]^A[x][2]^A[x][3]^A[x][4]

        for x in range(5):
            D[x] = C[(x-1)%5]^self.rot(C[(x+1)%5],1)

        for x in range(5):
            for y in range(5):
                A[x][y] = A[x][y]^D[x]

        #Rho and Pi steps
        for x in range(5):
          for y in range(5):
                B[y][(2*x+3*y)%5] = self.rot(A[x][y], self.r[x][y])

        #Chi step
        for x in range(5):
            for y in range(5):
                A[x][y] = B[x][y]^((~B[(x+1)%5][y]) & B[(x+2)%5][y])

        #Iota step
        A[0][0] = A[0][0]^RCfixed

        return A

    def KeccakF(self,A, verbose=False):
        """Perform Keccak-f function on the state A

        A: 5×5 matrix containing the state
        verbose: a boolean flag activating the printing of intermediate computations
        """

        if verbose:
            self.printState(A,"Before first round")

        for i in range(self.nr):
            #NB: result is truncated to lane size
            A = self.Round(A,self.RC[i]%(1<<self.w))

            if verbose:
                  self.printState(A,"Satus end of round #%d/%d" % (i+1,self.nr))

        return A

    def appendBit(self, M, bit):
        """Append a bit to M

        M: message pair (length in bits, string of hex characters ('9AFC...'))
        bit: 0 or 1
        Example: appendBit([7, '30'],1) returns [8,'B0']
        Example: appendBit([8, '30'],1) returns [9,'3001']
        """
        [my_string_length, my_string]=M
        if my_string_length>(len(my_string)//2*8):
            raise KeccakError.KeccakError("the string is too short to contain the number of bits announced")
        if ((my_string_length%8) == 0):
            my_string = my_string[0:my_string_length//8*2] + "%02X" % bit
            my_string_length = my_string_length + 1
        else:
            nr_bytes_filled = my_string_length//8
            nbr_bits_filled = my_string_length%8
            my_byte = int(my_string[nr_bytes_filled*2:nr_bytes_filled*2+2],16)
            my_byte = my_byte + bit*(2**(nbr_bits_filled))
            my_byte = "%02X" % my_byte
            my_string = my_string[0:nr_bytes_filled*2] + my_byte
            my_string_length = my_string_length + 1
        return [my_string_length, my_string]

    def appendDelimitedSuffix(self, M, suffix):
        """Append a delimited suffix to M

        M: message pair (length in bits, string of hex characters ('9AFC...'))
        suffix: integer coding a string of 0 to 7 bits, from LSB to MSB, delimited by a bit 1 at MSB
        Example: appendDelimitedSuffix([3, '00'], 0x06) returns [5, '10']
        Example: appendDelimitedSuffix([3, '00'], 0x1F) returns [7, '78']
        Example: appendDelimitedSuffix([8, '00'], 0x06) returns [10, '0002']
        Example: appendDelimitedSuffix([8, '00'], 0x1F) returns [12, '000F']
        """
        if (suffix == 0):
            raise KeccakError.KeccakError("the delimited suffix must not be zero")
        while(suffix != 1):
            M = self.appendBit(M, suffix%2)
            suffix = suffix//2
        return M

    def delimitedSuffixInBinary(self, delimitedSuffix):
        binary = ''
        while(delimitedSuffix != 1):
            binary = binary + ('%d' % (delimitedSuffix%2))
            delimitedSuffix = delimitedSuffix//2
        return binary

    ### Padding rule

    def pad10star1(self, M, n):
        """Pad M with the pad10*1 padding rule to reach a length multiple of r bits

        M: message pair (length in bits, string of hex characters ('9AFC...')
        n: length in bits (must be a multiple of 8)
        Example: pad10star1([60, 'BA594E0FB9EBBD03'],8) returns 'BA594E0FB9EBBD93'
        """

        [my_string_length, my_string]=M

        # Check the parameter n
        if n%8!=0:
            raise KeccakError.KeccakError("n must be a multiple of 8")

        # Check the length of the provided string
        if len(my_string)%2!=0:
            raise KeccakError.KeccakError("there must be an even number of digits")
        if my_string_length>(len(my_string)//2*8):
            raise KeccakError.KeccakError("the string is too short to contain the number of bits announced")

        nr_bytes_filled=my_string_length//8
        nbr_bits_filled=my_string_length%8
        l = my_string_length % n
        if ((n-8) <= l <= (n-2)):
            if (nbr_bits_filled == 0):
                my_byte = 0
            else:
                my_byte=int(my_string[nr_bytes_filled*2:nr_bytes_filled*2+2],16)
            my_byte=my_byte+2**(nbr_bits_filled)+2**7
            my_byte="%02X" % my_byte
            my_string=my_string[0:nr_bytes_filled*2]+my_byte
        else:
            if (nbr_bits_filled == 0):
                my_byte = 0
            else:
                my_byte=int(my_string[nr_bytes_filled*2:nr_bytes_filled*2+2],16)
            my_byte=my_byte+2**(nbr_bits_filled)
            my_byte="%02X" % my_byte
            my_string=my_string[0:nr_bytes_filled*2]+my_byte
            while((8*len(my_string)//2)%n < (n-8)):
                my_string=my_string+'00'
            my_string = my_string+'80'

        return my_string

    def Keccak(self,M,r=1024,c=576,suffix=0x01,n=1024,verbose=False):
        """Compute the Keccak[r,c,d] sponge function on message M

        M: message pair (length in bits, string of hex characters ('9AFC...')
        r: bitrate in bits (defautl: 1024)
        c: capacity in bits (default: 576)
        suffix: the delimited suffix to append to all inputs (0x01 means none, 0x06 for SHA3-* and 0x1F for SHAKE*)
        n: length of output in bits (default: 1024),
        verbose: print the details of computations(default:False)
        """

        #Check the inputs
        if (r<0) or (r%8!=0):
            raise KeccakError.KeccakError('r must be a multiple of 8 in this implementation')
        if (n%8!=0):
            raise KeccakError.KeccakError('outputLength must be a multiple of 8')
        self.setB(r+c)

        if verbose:
            print("Create a Keccak[r=%d, c=%d] function with '%s' suffix" % (r,c,self.delimitedSuffixInBinary(suffix)))

        #Compute lane length (in bits)
        w=(r+c)//25

        # Initialisation of state
        S=[[0,0,0,0,0],
           [0,0,0,0,0],
           [0,0,0,0,0],
           [0,0,0,0,0],
           [0,0,0,0,0]]

        # Appending the suffix
        M = self.appendDelimitedSuffix(M, suffix)
        if verbose:
            print("After appending the suffix: ", M)

        #Padding of messages
        P = self.pad10star1(M, r)

        if verbose:
            print("String ready to be absorbed: %s (will be completed by %d x '00')" % (P, c//8))

        #Absorbing phase
        for i in range((len(P)*8//2)//r):
            Pi=self.convertStrToTable(P[i*(2*r//8):(i+1)*(2*r//8)]+'00'*(c//8))

            for y in range(5):
              for x in range(5):
                  S[x][y] = S[x][y]^Pi[x][y]
            S = self.KeccakF(S, verbose)

        if verbose:
            print("Value after absorption : %s" % (self.convertTableToStr(S)))

        #Squeezing phase
        Z = ''
        outputLength = n
        while outputLength>0:
            string=self.convertTableToStr(S)
            Z = Z + string[:r*2//8]
            outputLength -= r
            if outputLength>0:
                S = self.KeccakF(S, verbose)

            # NB: done by block of length r, could have to be cut if outputLength
            #     is not a multiple of r

        if verbose:
            print("Value after squeezing : %s" % (self.convertTableToStr(S)))

        return Z[:2*n//8]



#!/usr/bin/env python
#
# Electrum - lightweight Bitcoin client
# Copyright (C) 2011 thomasv@gitorious
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Note about US patent no 5892470: Here each word does not represent a given digit.
# Instead, the digit represented by a word is variable, it depends on the previous word.
#
# Copied 28 October 2016 from Electrum:
#     https://github.com/spesmilo/electrum/blob/master/lib/old_mnemonic.py
#     Most recent commit: 305843999eabdb4276109e3dc6b70bf21dbbc29a
#
# list of words from:
#     https://github.com/monero-project/monero/blob/master/src/mnemonics/english.h
#     Most recent commit: de0392685063d93dbdad3a6b1a2712eaf94dd51a

from __future__ import print_function
from binascii import crc32 as _crc32

words = [
    "abbey",
    "abducts",
    "ability",
    "ablaze",
    "abnormal",
    "abort",
    "abrasive",
    "absorb",
    "abyss",
    "academy",
    "aces",
    "aching",
    "acidic",
    "acoustic",
    "acquire",
    "across",
    "actress",
    "acumen",
    "adapt",
    "addicted",
    "adept",
    "adhesive",
    "adjust",
    "adopt",
    "adrenalin",
    "adult",
    "adventure",
    "aerial",
    "afar",
    "affair",
    "afield",
    "afloat",
    "afoot",
    "afraid",
    "after",
    "against",
    "agenda",
    "aggravate",
    "agile",
    "aglow",
    "agnostic",
    "agony",
    "agreed",
    "ahead",
    "aided",
    "ailments",
    "aimless",
    "airport",
    "aisle",
    "ajar",
    "akin",
    "alarms",
    "album",
    "alchemy",
    "alerts",
    "algebra",
    "alkaline",
    "alley",
    "almost",
    "aloof",
    "alpine",
    "already",
    "also",
    "altitude",
    "alumni",
    "always",
    "amaze",
    "ambush",
    "amended",
    "amidst",
    "ammo",
    "amnesty",
    "among",
    "amply",
    "amused",
    "anchor",
    "android",
    "anecdote",
    "angled",
    "ankle",
    "annoyed",
    "answers",
    "antics",
    "anvil",
    "anxiety",
    "anybody",
    "apart",
    "apex",
    "aphid",
    "aplomb",
    "apology",
    "apply",
    "apricot",
    "aptitude",
    "aquarium",
    "arbitrary",
    "archer",
    "ardent",
    "arena",
    "argue",
    "arises",
    "army",
    "around",
    "arrow",
    "arsenic",
    "artistic",
    "ascend",
    "ashtray",
    "aside",
    "asked",
    "asleep",
    "aspire",
    "assorted",
    "asylum",
    "athlete",
    "atlas",
    "atom",
    "atrium",
    "attire",
    "auburn",
    "auctions",
    "audio",
    "august",
    "aunt",
    "austere",
    "autumn",
    "avatar",
    "avidly",
    "avoid",
    "awakened",
    "awesome",
    "awful",
    "awkward",
    "awning",
    "awoken",
    "axes",
    "axis",
    "axle",
    "aztec",
    "azure",
    "baby",
    "bacon",
    "badge",
    "baffles",
    "bagpipe",
    "bailed",
    "bakery",
    "balding",
    "bamboo",
    "banjo",
    "baptism",
    "basin",
    "batch",
    "bawled",
    "bays",
    "because",
    "beer",
    "befit",
    "begun",
    "behind",
    "being",
    "below",
    "bemused",
    "benches",
    "berries",
    "bested",
    "betting",
    "bevel",
    "beware",
    "beyond",
    "bias",
    "bicycle",
    "bids",
    "bifocals",
    "biggest",
    "bikini",
    "bimonthly",
    "binocular",
    "biology",
    "biplane",
    "birth",
    "biscuit",
    "bite",
    "biweekly",
    "blender",
    "blip",
    "bluntly",
    "boat",
    "bobsled",
    "bodies",
    "bogeys",
    "boil",
    "boldly",
    "bomb",
    "border",
    "boss",
    "both",
    "bounced",
    "bovine",
    "bowling",
    "boxes",
    "boyfriend",
    "broken",
    "brunt",
    "bubble",
    "buckets",
    "budget",
    "buffet",
    "bugs",
    "building",
    "bulb",
    "bumper",
    "bunch",
    "business",
    "butter",
    "buying",
    "buzzer",
    "bygones",
    "byline",
    "bypass",
    "cabin",
    "cactus",
    "cadets",
    "cafe",
    "cage",
    "cajun",
    "cake",
    "calamity",
    "camp",
    "candy",
    "casket",
    "catch",
    "cause",
    "cavernous",
    "cease",
    "cedar",
    "ceiling",
    "cell",
    "cement",
    "cent",
    "certain",
    "chlorine",
    "chrome",
    "cider",
    "cigar",
    "cinema",
    "circle",
    "cistern",
    "citadel",
    "civilian",
    "claim",
    "click",
    "clue",
    "coal",
    "cobra",
    "cocoa",
    "code",
    "coexist",
    "coffee",
    "cogs",
    "cohesive",
    "coils",
    "colony",
    "comb",
    "cool",
    "copy",
    "corrode",
    "costume",
    "cottage",
    "cousin",
    "cowl",
    "criminal",
    "cube",
    "cucumber",
    "cuddled",
    "cuffs",
    "cuisine",
    "cunning",
    "cupcake",
    "custom",
    "cycling",
    "cylinder",
    "cynical",
    "dabbing",
    "dads",
    "daft",
    "dagger",
    "daily",
    "damp",
    "dangerous",
    "dapper",
    "darted",
    "dash",
    "dating",
    "dauntless",
    "dawn",
    "daytime",
    "dazed",
    "debut",
    "decay",
    "dedicated",
    "deepest",
    "deftly",
    "degrees",
    "dehydrate",
    "deity",
    "dejected",
    "delayed",
    "demonstrate",
    "dented",
    "deodorant",
    "depth",
    "desk",
    "devoid",
    "dewdrop",
    "dexterity",
    "dialect",
    "dice",
    "diet",
    "different",
    "digit",
    "dilute",
    "dime",
    "dinner",
    "diode",
    "diplomat",
    "directed",
    "distance",
    "ditch",
    "divers",
    "dizzy",
    "doctor",
    "dodge",
    "does",
    "dogs",
    "doing",
    "dolphin",
    "domestic",
    "donuts",
    "doorway",
    "dormant",
    "dosage",
    "dotted",
    "double",
    "dove",
    "down",
    "dozen",
    "dreams",
    "drinks",
    "drowning",
    "drunk",
    "drying",
    "dual",
    "dubbed",
    "duckling",
    "dude",
    "duets",
    "duke",
    "dullness",
    "dummy",
    "dunes",
    "duplex",
    "duration",
    "dusted",
    "duties",
    "dwarf",
    "dwelt",
    "dwindling",
    "dying",
    "dynamite",
    "dyslexic",
    "each",
    "eagle",
    "earth",
    "easy",
    "eating",
    "eavesdrop",
    "eccentric",
    "echo",
    "eclipse",
    "economics",
    "ecstatic",
    "eden",
    "edgy",
    "edited",
    "educated",
    "eels",
    "efficient",
    "eggs",
    "egotistic",
    "eight",
    "either",
    "eject",
    "elapse",
    "elbow",
    "eldest",
    "eleven",
    "elite",
    "elope",
    "else",
    "eluded",
    "emails",
    "ember",
    "emerge",
    "emit",
    "emotion",
    "empty",
    "emulate",
    "energy",
    "enforce",
    "enhanced",
    "enigma",
    "enjoy",
    "enlist",
    "enmity",
    "enough",
    "enraged",
    "ensign",
    "entrance",
    "envy",
    "epoxy",
    "equip",
    "erase",
    "erected",
    "erosion",
    "error",
    "eskimos",
    "espionage",
    "essential",
    "estate",
    "etched",
    "eternal",
    "ethics",
    "etiquette",
    "evaluate",
    "evenings",
    "evicted",
    "evolved",
    "examine",
    "excess",
    "exhale",
    "exit",
    "exotic",
    "exquisite",
    "extra",
    "exult",
    "fabrics",
    "factual",
    "fading",
    "fainted",
    "faked",
    "fall",
    "family",
    "fancy",
    "farming",
    "fatal",
    "faulty",
    "fawns",
    "faxed",
    "fazed",
    "feast",
    "february",
    "federal",
    "feel",
    "feline",
    "females",
    "fences",
    "ferry",
    "festival",
    "fetches",
    "fever",
    "fewest",
    "fiat",
    "fibula",
    "fictional",
    "fidget",
    "fierce",
    "fifteen",
    "fight",
    "films",
    "firm",
    "fishing",
    "fitting",
    "five",
    "fixate",
    "fizzle",
    "fleet",
    "flippant",
    "flying",
    "foamy",
    "focus",
    "foes",
    "foggy",
    "foiled",
    "folding",
    "fonts",
    "foolish",
    "fossil",
    "fountain",
    "fowls",
    "foxes",
    "foyer",
    "framed",
    "friendly",
    "frown",
    "fruit",
    "frying",
    "fudge",
    "fuel",
    "fugitive",
    "fully",
    "fuming",
    "fungal",
    "furnished",
    "fuselage",
    "future",
    "fuzzy",
    "gables",
    "gadget",
    "gags",
    "gained",
    "galaxy",
    "gambit",
    "gang",
    "gasp",
    "gather",
    "gauze",
    "gave",
    "gawk",
    "gaze",
    "gearbox",
    "gecko",
    "geek",
    "gels",
    "gemstone",
    "general",
    "geometry",
    "germs",
    "gesture",
    "getting",
    "geyser",
    "ghetto",
    "ghost",
    "giant",
    "giddy",
    "gifts",
    "gigantic",
    "gills",
    "gimmick",
    "ginger",
    "girth",
    "giving",
    "glass",
    "gleeful",
    "glide",
    "gnaw",
    "gnome",
    "goat",
    "goblet",
    "godfather",
    "goes",
    "goggles",
    "going",
    "goldfish",
    "gone",
    "goodbye",
    "gopher",
    "gorilla",
    "gossip",
    "gotten",
    "gourmet",
    "governing",
    "gown",
    "greater",
    "grunt",
    "guarded",
    "guest",
    "guide",
    "gulp",
    "gumball",
    "guru",
    "gusts",
    "gutter",
    "guys",
    "gymnast",
    "gypsy",
    "gyrate",
    "habitat",
    "hacksaw",
    "haggled",
    "hairy",
    "hamburger",
    "happens",
    "hashing",
    "hatchet",
    "haunted",
    "having",
    "hawk",
    "haystack",
    "hazard",
    "hectare",
    "hedgehog",
    "heels",
    "hefty",
    "height",
    "hemlock",
    "hence",
    "heron",
    "hesitate",
    "hexagon",
    "hickory",
    "hiding",
    "highway",
    "hijack",
    "hiker",
    "hills",
    "himself",
    "hinder",
    "hippo",
    "hire",
    "history",
    "hitched",
    "hive",
    "hoax",
    "hobby",
    "hockey",
    "hoisting",
    "hold",
    "honked",
    "hookup",
    "hope",
    "hornet",
    "hospital",
    "hotel",
    "hounded",
    "hover",
    "howls",
    "hubcaps",
    "huddle",
    "huge",
    "hull",
    "humid",
    "hunter",
    "hurried",
    "husband",
    "huts",
    "hybrid",
    "hydrogen",
    "hyper",
    "iceberg",
    "icing",
    "icon",
    "identity",
    "idiom",
    "idled",
    "idols",
    "igloo",
    "ignore",
    "iguana",
    "illness",
    "imagine",
    "imbalance",
    "imitate",
    "impel",
    "inactive",
    "inbound",
    "incur",
    "industrial",
    "inexact",
    "inflamed",
    "ingested",
    "initiate",
    "injury",
    "inkling",
    "inline",
    "inmate",
    "innocent",
    "inorganic",
    "input",
    "inquest",
    "inroads",
    "insult",
    "intended",
    "inundate",
    "invoke",
    "inwardly",
    "ionic",
    "irate",
    "iris",
    "irony",
    "irritate",
    "island",
    "isolated",
    "issued",
    "italics",
    "itches",
    "items",
    "itinerary",
    "itself",
    "ivory",
    "jabbed",
    "jackets",
    "jaded",
    "jagged",
    "jailed",
    "jamming",
    "january",
    "jargon",
    "jaunt",
    "javelin",
    "jaws",
    "jazz",
    "jeans",
    "jeers",
    "jellyfish",
    "jeopardy",
    "jerseys",
    "jester",
    "jetting",
    "jewels",
    "jigsaw",
    "jingle",
    "jittery",
    "jive",
    "jobs",
    "jockey",
    "jogger",
    "joining",
    "joking",
    "jolted",
    "jostle",
    "journal",
    "joyous",
    "jubilee",
    "judge",
    "juggled",
    "juicy",
    "jukebox",
    "july",
    "jump",
    "junk",
    "jury",
    "justice",
    "juvenile",
    "kangaroo",
    "karate",
    "keep",
    "kennel",
    "kept",
    "kernels",
    "kettle",
    "keyboard",
    "kickoff",
    "kidneys",
    "king",
    "kiosk",
    "kisses",
    "kitchens",
    "kiwi",
    "knapsack",
    "knee",
    "knife",
    "knowledge",
    "knuckle",
    "koala",
    "laboratory",
    "ladder",
    "lagoon",
    "lair",
    "lakes",
    "lamb",
    "language",
    "laptop",
    "large",
    "last",
    "later",
    "launching",
    "lava",
    "lawsuit",
    "layout",
    "lazy",
    "lectures",
    "ledge",
    "leech",
    "left",
    "legion",
    "leisure",
    "lemon",
    "lending",
    "leopard",
    "lesson",
    "lettuce",
    "lexicon",
    "liar",
    "library",
    "licks",
    "lids",
    "lied",
    "lifestyle",
    "light",
    "likewise",
    "lilac",
    "limits",
    "linen",
    "lion",
    "lipstick",
    "liquid",
    "listen",
    "lively",
    "loaded",
    "lobster",
    "locker",
    "lodge",
    "lofty",
    "logic",
    "loincloth",
    "long",
    "looking",
    "lopped",
    "lordship",
    "losing",
    "lottery",
    "loudly",
    "love",
    "lower",
    "loyal",
    "lucky",
    "luggage",
    "lukewarm",
    "lullaby",
    "lumber",
    "lunar",
    "lurk",
    "lush",
    "luxury",
    "lymph",
    "lynx",
    "lyrics",
    "macro",
    "madness",
    "magically",
    "mailed",
    "major",
    "makeup",
    "malady",
    "mammal",
    "maps",
    "masterful",
    "match",
    "maul",
    "maverick",
    "maximum",
    "mayor",
    "maze",
    "meant",
    "mechanic",
    "medicate",
    "meeting",
    "megabyte",
    "melting",
    "memoir",
    "menu",
    "merger",
    "mesh",
    "metro",
    "mews",
    "mice",
    "midst",
    "mighty",
    "mime",
    "mirror",
    "misery",
    "mittens",
    "mixture",
    "moat",
    "mobile",
    "mocked",
    "mohawk",
    "moisture",
    "molten",
    "moment",
    "money",
    "moon",
    "mops",
    "morsel",
    "mostly",
    "motherly",
    "mouth",
    "movement",
    "mowing",
    "much",
    "muddy",
    "muffin",
    "mugged",
    "mullet",
    "mumble",
    "mundane",
    "muppet",
    "mural",
    "musical",
    "muzzle",
    "myriad",
    "mystery",
    "myth",
    "nabbing",
    "nagged",
    "nail",
    "names",
    "nanny",
    "napkin",
    "narrate",
    "nasty",
    "natural",
    "nautical",
    "navy",
    "nearby",
    "necklace",
    "needed",
    "negative",
    "neither",
    "neon",
    "nephew",
    "nerves",
    "nestle",
    "network",
    "neutral",
    "never",
    "newt",
    "nexus",
    "nibs",
    "niche",
    "niece",
    "nifty",
    "nightly",
    "nimbly",
    "nineteen",
    "nirvana",
    "nitrogen",
    "nobody",
    "nocturnal",
    "nodes",
    "noises",
    "nomad",
    "noodles",
    "northern",
    "nostril",
    "noted",
    "nouns",
    "novelty",
    "nowhere",
    "nozzle",
    "nuance",
    "nucleus",
    "nudged",
    "nugget",
    "nuisance",
    "null",
    "number",
    "nuns",
    "nurse",
    "nutshell",
    "nylon",
    "oaks",
    "oars",
    "oasis",
    "oatmeal",
    "obedient",
    "object",
    "obliged",
    "obnoxious",
    "observant",
    "obtains",
    "obvious",
    "occur",
    "ocean",
    "october",
    "odds",
    "odometer",
    "offend",
    "often",
    "oilfield",
    "ointment",
    "okay",
    "older",
    "olive",
    "olympics",
    "omega",
    "omission",
    "omnibus",
    "onboard",
    "oncoming",
    "oneself",
    "ongoing",
    "onion",
    "online",
    "onslaught",
    "onto",
    "onward",
    "oozed",
    "opacity",
    "opened",
    "opposite",
    "optical",
    "opus",
    "orange",
    "orbit",
    "orchid",
    "orders",
    "organs",
    "origin",
    "ornament",
    "orphans",
    "oscar",
    "ostrich",
    "otherwise",
    "otter",
    "ouch",
    "ought",
    "ounce",
    "ourselves",
    "oust",
    "outbreak",
    "oval",
    "oven",
    "owed",
    "owls",
    "owner",
    "oxidant",
    "oxygen",
    "oyster",
    "ozone",
    "pact",
    "paddles",
    "pager",
    "pairing",
    "palace",
    "pamphlet",
    "pancakes",
    "paper",
    "paradise",
    "pastry",
    "patio",
    "pause",
    "pavements",
    "pawnshop",
    "payment",
    "peaches",
    "pebbles",
    "peculiar",
    "pedantic",
    "peeled",
    "pegs",
    "pelican",
    "pencil",
    "people",
    "pepper",
    "perfect",
    "pests",
    "petals",
    "phase",
    "pheasants",
    "phone",
    "phrases",
    "physics",
    "piano",
    "picked",
    "pierce",
    "pigment",
    "piloted",
    "pimple",
    "pinched",
    "pioneer",
    "pipeline",
    "pirate",
    "pistons",
    "pitched",
    "pivot",
    "pixels",
    "pizza",
    "playful",
    "pledge",
    "pliers",
    "plotting",
    "plus",
    "plywood",
    "poaching",
    "pockets",
    "podcast",
    "poetry",
    "point",
    "poker",
    "polar",
    "ponies",
    "pool",
    "popular",
    "portents",
    "possible",
    "potato",
    "pouch",
    "poverty",
    "powder",
    "pram",
    "present",
    "pride",
    "problems",
    "pruned",
    "prying",
    "psychic",
    "public",
    "puck",
    "puddle",
    "puffin",
    "pulp",
    "pumpkins",
    "punch",
    "puppy",
    "purged",
    "push",
    "putty",
    "puzzled",
    "pylons",
    "pyramid",
    "python",
    "queen",
    "quick",
    "quote",
    "rabbits",
    "racetrack",
    "radar",
    "rafts",
    "rage",
    "railway",
    "raking",
    "rally",
    "ramped",
    "randomly",
    "rapid",
    "rarest",
    "rash",
    "rated",
    "ravine",
    "rays",
    "razor",
    "react",
    "rebel",
    "recipe",
    "reduce",
    "reef",
    "refer",
    "regular",
    "reheat",
    "reinvest",
    "rejoices",
    "rekindle",
    "relic",
    "remedy",
    "renting",
    "reorder",
    "repent",
    "request",
    "reruns",
    "rest",
    "return",
    "reunion",
    "revamp",
    "rewind",
    "rhino",
    "rhythm",
    "ribbon",
    "richly",
    "ridges",
    "rift",
    "rigid",
    "rims",
    "ringing",
    "riots",
    "ripped",
    "rising",
    "ritual",
    "river",
    "roared",
    "robot",
    "rockets",
    "rodent",
    "rogue",
    "roles",
    "romance",
    "roomy",
    "roped",
    "roster",
    "rotate",
    "rounded",
    "rover",
    "rowboat",
    "royal",
    "ruby",
    "rudely",
    "ruffled",
    "rugged",
    "ruined",
    "ruling",
    "rumble",
    "runway",
    "rural",
    "rustled",
    "ruthless",
    "sabotage",
    "sack",
    "sadness",
    "safety",
    "saga",
    "sailor",
    "sake",
    "salads",
    "sample",
    "sanity",
    "sapling",
    "sarcasm",
    "sash",
    "satin",
    "saucepan",
    "saved",
    "sawmill",
    "saxophone",
    "sayings",
    "scamper",
    "scenic",
    "school",
    "science",
    "scoop",
    "scrub",
    "scuba",
    "seasons",
    "second",
    "sedan",
    "seeded",
    "segments",
    "seismic",
    "selfish",
    "semifinal",
    "sensible",
    "september",
    "sequence",
    "serving",
    "session",
    "setup",
    "seventh",
    "sewage",
    "shackles",
    "shelter",
    "shipped",
    "shocking",
    "shrugged",
    "shuffled",
    "shyness",
    "siblings",
    "sickness",
    "sidekick",
    "sieve",
    "sifting",
    "sighting",
    "silk",
    "simplest",
    "sincerely",
    "sipped",
    "siren",
    "situated",
    "sixteen",
    "sizes",
    "skater",
    "skew",
    "skirting",
    "skulls",
    "skydive",
    "slackens",
    "sleepless",
    "slid",
    "slower",
    "slug",
    "smash",
    "smelting",
    "smidgen",
    "smog",
    "smuggled",
    "snake",
    "sneeze",
    "sniff",
    "snout",
    "snug",
    "soapy",
    "sober",
    "soccer",
    "soda",
    "software",
    "soggy",
    "soil",
    "solved",
    "somewhere",
    "sonic",
    "soothe",
    "soprano",
    "sorry",
    "southern",
    "sovereign",
    "sowed",
    "soya",
    "space",
    "speedy",
    "sphere",
    "spiders",
    "splendid",
    "spout",
    "sprig",
    "spud",
    "spying",
    "square",
    "stacking",
    "stellar",
    "stick",
    "stockpile",
    "strained",
    "stunning",
    "stylishly",
    "subtly",
    "succeed",
    "suddenly",
    "suede",
    "suffice",
    "sugar",
    "suitcase",
    "sulking",
    "summon",
    "sunken",
    "superior",
    "surfer",
    "sushi",
    "suture",
    "swagger",
    "swept",
    "swiftly",
    "sword",
    "swung",
    "syllabus",
    "symptoms",
    "syndrome",
    "syringe",
    "system",
    "taboo",
    "tacit",
    "tadpoles",
    "tagged",
    "tail",
    "taken",
    "talent",
    "tamper",
    "tanks",
    "tapestry",
    "tarnished",
    "tasked",
    "tattoo",
    "taunts",
    "tavern",
    "tawny",
    "taxi",
    "teardrop",
    "technical",
    "tedious",
    "teeming",
    "tell",
    "template",
    "tender",
    "tepid",
    "tequila",
    "terminal",
    "testing",
    "tether",
    "textbook",
    "thaw",
    "theatrics",
    "thirsty",
    "thorn",
    "threaten",
    "thumbs",
    "thwart",
    "ticket",
    "tidy",
    "tiers",
    "tiger",
    "tilt",
    "timber",
    "tinted",
    "tipsy",
    "tirade",
    "tissue",
    "titans",
    "toaster",
    "tobacco",
    "today",
    "toenail",
    "toffee",
    "together",
    "toilet",
    "token",
    "tolerant",
    "tomorrow",
    "tonic",
    "toolbox",
    "topic",
    "torch",
    "tossed",
    "total",
    "touchy",
    "towel",
    "toxic",
    "toyed",
    "trash",
    "trendy",
    "tribal",
    "trolling",
    "truth",
    "trying",
    "tsunami",
    "tubes",
    "tucks",
    "tudor",
    "tuesday",
    "tufts",
    "tugs",
    "tuition",
    "tulips",
    "tumbling",
    "tunnel",
    "turnip",
    "tusks",
    "tutor",
    "tuxedo",
    "twang",
    "tweezers",
    "twice",
    "twofold",
    "tycoon",
    "typist",
    "tyrant",
    "ugly",
    "ulcers",
    "ultimate",
    "umbrella",
    "umpire",
    "unafraid",
    "unbending",
    "uncle",
    "under",
    "uneven",
    "unfit",
    "ungainly",
    "unhappy",
    "union",
    "unjustly",
    "unknown",
    "unlikely",
    "unmask",
    "unnoticed",
    "unopened",
    "unplugs",
    "unquoted",
    "unrest",
    "unsafe",
    "until",
    "unusual",
    "unveil",
    "unwind",
    "unzip",
    "upbeat",
    "upcoming",
    "update",
    "upgrade",
    "uphill",
    "upkeep",
    "upload",
    "upon",
    "upper",
    "upright",
    "upstairs",
    "uptight",
    "upwards",
    "urban",
    "urchins",
    "urgent",
    "usage",
    "useful",
    "usher",
    "using",
    "usual",
    "utensils",
    "utility",
    "utmost",
    "utopia",
    "uttered",
    "vacation",
    "vague",
    "vain",
    "value",
    "vampire",
    "vane",
    "vapidly",
    "vary",
    "vastness",
    "vats",
    "vaults",
    "vector",
    "veered",
    "vegan",
    "vehicle",
    "vein",
    "velvet",
    "venomous",
    "verification",
    "vessel",
    "veteran",
    "vexed",
    "vials",
    "vibrate",
    "victim",
    "video",
    "viewpoint",
    "vigilant",
    "viking",
    "village",
    "vinegar",
    "violin",
    "vipers",
    "virtual",
    "visited",
    "vitals",
    "vivid",
    "vixen",
    "vocal",
    "vogue",
    "voice",
    "volcano",
    "vortex",
    "voted",
    "voucher",
    "vowels",
    "voyage",
    "vulture",
    "wade",
    "waffle",
    "wagtail",
    "waist",
    "waking",
    "wallets",
    "wanted",
    "warped",
    "washing",
    "water",
    "waveform",
    "waxing",
    "wayside",
    "weavers",
    "website",
    "wedge",
    "weekday",
    "weird",
    "welders",
    "went",
    "wept",
    "were",
    "western",
    "wetsuit",
    "whale",
    "when",
    "whipped",
    "whole",
    "wickets",
    "width",
    "wield",
    "wife",
    "wiggle",
    "wildly",
    "winter",
    "wipeout",
    "wiring",
    "wise",
    "withdrawn",
    "wives",
    "wizard",
    "wobbly",
    "woes",
    "woken",
    "wolf",
    "womanly",
    "wonders",
    "woozy",
    "worry",
    "wounded",
    "woven",
    "wrap",
    "wrist",
    "wrong",
    "yacht",
    "yahoo",
    "yanks",
    "yard",
    "yawning",
    "yearbook",
    "yellow",
    "yesterday",
    "yeti",
    "yields",
    "yodel",
    "yoga",
    "younger",
    "yoyo",
    "zapped",
    "zeal",
    "zebra",
    "zero",
    "zesty",
    "zigzags",
    "zinger",
    "zippers",
    "zodiac",
    "zombie",
    "zones",
    "zoom"
]

n = 1626

def mn_checksum(wlist):
    '''Given a mnemonic seed word list, return a string of the seed checksum.'''
    if len(wlist) > 13:
        wlist = wlist[:24]
    else:
        wlist = wlist[:12]
    wstr = "".join(word[:3] for word in wlist)
    z = ((_crc32(wstr.encode()) & 0xffffffff) ^ 0xffffffff ) >> 0
    z2 = ((z ^ 0xffffffff) >> 0) % len(wlist)
    return wlist[z2]

def mn_validate_checksum(wlist):
    '''Given a mnemonic seed word list, check if checksum word is valid.
    Returns boolean value.
    '''
    return True if mn_checksum(wlist) == wlist[-1] else False

def mn_swap_endian(word):
    return "".join([word[i:i+2] for i in [6, 4, 2, 0]])

def mn_encode(message):
    '''Given a hexadecimal key string, return it's mnemonic representation.'''
    assert len(message)%8 == 0
    out = []
    for i in range(len(message)//8):
        word = mn_swap_endian(message[8*i:8*i+8])
        x = int(word, 16)
        w1 = x%n
        w2 = (x//n + w1)%n
        w3 = (x//n//n + w2)%n
        out += [words[w1], words[w2], words[w3]]
    return out

def mn_decode(wlist):
    '''Given a mnemonic word list, return a hexadecimal encoded string.'''
    out = ""
    for i in range(len(wlist)//3):
        word1, word2, word3 = wlist[3*i:3*i+3]
        w1 = words.index(word1)
        w2 = words.index(word2)%n
        w3 = words.index(word3)%n
        x = w1 + n*((w2-w1)%n) + n*n*((w3-w2)%n)
        out += mn_swap_endian("%08x"%x)
    return out


if __name__ == "__main__":
    import sys
    if len(sys.argv) == 1:
        print("I need arguments: a hex string to encode, or a list of words to decode")
    elif len(sys.argv) == 2:
        print(" ".join(mn_encode(sys.argv[1])))
    else:
        print(mn_decode(sys.argv[1:]))



#from __future__ import absolute_import
import os
import sys
sys.path.insert(0, os.path.abspath('..'))

import moneropy.mnemonic
import moneropy.utils
import moneropy.base58
import moneropy.cryptonote
import moneropy.account



import unittest
from testdata import valid_seeds, valid_sks, valid_vks, valid_addrs, addr_vers, valid_addr_pubsks, valid_addr_pubvks
from context import moneropy

acc = moneropy.account

class TestAccount(unittest.TestCase):

    def test_get_view_key(self):
        def T(sk, vk):
            self.assertEqual(acc.get_view_key(sk), vk)

        for i in range(len(valid_sks)):
            T(valid_sks[i], valid_vks[i])

    def test_account_from_seed(self):
        def T(seed, sk, vk, addr):
            sk_t, vk_t, addr_t = acc.account_from_seed(seed)
            self.assertEqual(sk_t, sk)
            self.assertEqual(vk_t, vk)
            self.assertEqual(addr_t, addr)

        for i in range(len(valid_seeds)):
            T(valid_seeds[i], valid_sks[i], valid_vks[i], valid_addrs[i])

    def test_account_from_spend_key(self):
        def T(sk, vk, addr):
            sk_t, vk_t, addr_t = acc.account_from_spend_key(sk)
            self.assertEqual(sk_t, sk)
            self.assertEqual(vk_t, vk)
            self.assertEqual(addr_t, addr)

        for i in range(len(valid_sks)):
            T(valid_sks[i], valid_vks[i], valid_addrs[i])

    def test_decode_addr(self):
        def T(addr, vers, pubsk, pubvk):
            vers_, pubsk_, pubvk_ = acc.decode_addr(addr)
            self.assertEqual(vers_, vers)
            self.assertEqual(pubsk_, pubsk)
            self.assertEqual(pubvk_, pubvk)

        for i in range(len(valid_addrs)):
            T(valid_addrs[i], addr_vers, valid_addr_pubsks[i], valid_addr_pubvks[i])

    def test_encode_addr(self):
        def T(vers, pubsk, pubvk, addr):
            self.assertEqual(acc.encode_addr(vers, pubsk, pubvk), addr)

        for i in range(len(valid_addr_pubsks)):
            T(addr_vers, valid_addr_pubsks[i], valid_addr_pubvks[i], valid_addrs[i])

    def test_check_address_viewkey(self):
        def T(addr, privViewKey):
            self.assertTrue(acc.check_address_viewkey(addr, privViewKey))

        def T2(addr, privViewKey):
            self.assertFalse(acc.check_address_viewkey(addr, privViewKey))

        # Test valid view keys
        for i in range(len(valid_addrs)):
            T(valid_addrs[i], valid_vks[i])

        # Test invalid view keys (using mis-matched vk/addr pair)
        for i in range(len(valid_addrs)):
            j = (i+1) % len(valid_addrs)
            T2(valid_addrs[i], valid_vks[j])

if __name__ == '__main__':
    unittest.main()



import unittest
from testdata import valid_sks, hashed_valid_sks, reduced_hashed_valid_sks, public_from_valid_sks
from context import moneropy

cn = moneropy.cryptonote

class TestCryptonote(unittest.TestCase):

    def test_cn_fast_hash(self):
        def T(sk, hashed_sk):
            self.assertEqual(cn.cn_fast_hash(sk), hashed_sk)

        for i in range(len(valid_sks)):
            T(valid_sks[i], hashed_valid_sks[i])

    def test_sc_reduce(self):
        def T(hashed_sk, red_hashed_sk):
            self.assertEqual(cn.sc_reduce(hashed_sk), red_hashed_sk)

        for i in range(len(hashed_valid_sks)):
            T(hashed_valid_sks[i], reduced_hashed_valid_sks[i])


    def test_public_from_secret(self):
        def T(sk, public):
            self.assertEqual(cn.public_from_secret(sk), public)

        for i in range(len(valid_sks)):
            T(valid_sks[i], public_from_valid_sks[i])



if __name__ == '__main__':
    unittest.main()



import unittest
from testdata import hexes, ints, extra_hex, extra_bin, extra_pub, extra_payid
from context import moneropy

utils = moneropy.utils

class TestUtils(unittest.TestCase):

    def test_hexStrToInt(self):
        def T(h, i):
            self.assertEqual(utils.hexStrToInt(h), i)

        for i in range(len(hexes)):
            T(hexes[i], ints[i])

    def test_intToHexStr(self):
        def T(i, h):
            self.assertEqual(utils.intToHexStr(i), h)

        for i in range(len(ints)):
            T(ints[i], hexes[i])

    def test_parse_extra_hex(self):
        def T(h, pub, pid):
            pub_, pid_ = utils.parse_extra_hex(h)
            self.assertEqual(pub_, pub)
            self.assertEqual(pid_, pid)

        for i in range(len(extra_hex)):
            T(extra_hex[i], extra_pub[i], extra_payid[i])

    def test_parse_extra_bin(self):
        def T(b, pub, pid):
            pub_, pid_ = utils.parse_extra_bin(b)
            self.assertEqual(pub_, pub)
            self.assertEqual(pid_, pid)

        for i in range(len(extra_bin)):
            T(extra_bin[i], extra_pub[i], extra_payid[i])


if __name__ == '__main__':
    unittest.main()



# MoneroPy - A python toolbox for Monero
# Copyright (C) 2016 The MoneroPy Developers.
#
# MoneroPy is released under the BSD 3-Clause license. Use and redistribution of
# this software is subject to the license terms in the LICENSE file found in the
# top-level directory of this distribution.

from . import account as _acc
from . import base58 as _b58
from . import cryptonote as _cn
from . import utils as _utils

KEY_SIZE = 32
HASH_SIZE = 32

def valid_hex(h):
    is_valid = True
    try:
        i = int(h, 16)
    except:
        is_valid = False
    return is_valid

def random_scalar():
    return _cn.sc_reduce32(_utils.gen_random_hex())

def hash_to_scalar(buf):
    return _cn.sc_reduce32(_cn.cn_fast_hash(buf))

def sc_mulsub(a, b, c):
    a_minus_bc = _utils.hexStrToInt(c) - _utils.hexStrToInt(a) * _utils.hexStrToInt(b)
    return _utils.intToHexStr(a_minus_bc % _cn.l)

def generate_signature(prefix_hash, pub, sec):
    err_msg = ""
    if len(sec) != KEY_SIZE*2 or valid_hex(sec) != True:
        err_msg = "Invalid secret key"
        return err_msg
    if len(prefix_hash) != HASH_SIZE*2 or valid_hex(prefix_hash) != True:
        err_msg = "Invalid prefix hash"
        return err_msg
    try:
        comm = _cn.public_from_secret(random_scalar())
        c = hash_to_scalar(prefix_hash + pub + comm)
        r = sc_mulsub(c, sec, k)
    except:
        err_msg = "There was a problem generating a signature."
        return err_msg
    return c + r

def sign_message(msg, sec, address, key_type):
    err = 0
    # Verify valid address:
    vers, psk, pvk = _acc.decode_addr(address)
    if vers == "Invalid Address":
        err = 1
        return_message = 'Address validation failed! Please check it and try again.'
        return return_message, err
    # Verify secret key:
    if key_type == "Private Spend Key":
        if _cn.public_from_secret(sec) != psk:
            err = 2
            return_message = 'Your Private Spendkey does not belong to this address!'
            return return_message, err
    else:
        if _cn.public_from_secret(sec) != pvk:
            err = 3
            return_message = 'Your Private Viewkey does not belong to this address!'
            return return_message, err
    # Sign message:
    msgHash = _cn.keccak_256(msg.encode())
    sig = generate_signature(msgHash, psk, sec)
    sig = "SigV1" + _b58.encode(sig)
    return_message = ("-----BEGIN MONERO SIGNED MESSAGE-----", msg,
        "Address: " + address, "-----BEGIN SIGNATURE-----", sig,
        "-----END MONERO SIGNED MESSAGE-----")
    return return_message, err



import unittest
from testdata import valid_addrs, decoded_addrs
from context import moneropy

b58 = moneropy.base58

class TestBase58(unittest.TestCase):

    def test_encode(self):
        def T(dec_addr, addr):
            self.assertEqual(b58.encode(dec_addr), addr)

        for i in range(len(valid_addrs)):
            T(decoded_addrs[i], valid_addrs[i])

    def test_decode(self):
        def T(addr, dec_addr):
            self.assertEqual(b58.decode(addr), dec_addr)

        for i in range(len(valid_addrs)):
            T(valid_addrs[i], decoded_addrs[i])


if __name__ == '__main__':
    unittest.main()



# MoneroPy - A python toolbox for Monero
# Copyright (C) 2016 The MoneroPy Developers.
#
# MoneroPy is released under the BSD 3-Clause license. Use and redistribution of
# this software is subject to the license terms in the LICENSE file found in the
# top-level directory of this distribution.

from . import mnemonic as _mn
from . import cryptonote as _cn
from . import base58 as _b58
from . import utils as _utils

ADDRESS_VERSION = "12"

def get_view_key(sk):
    '''Computes view key for input secret key.'''
    return _cn.sc_reduce(_cn.cn_fast_hash(sk))

def encode_addr(version, publicSpendKey, publicViewKey):
    '''Given address version and public spend and view keys, derive address.'''
    data = version + publicSpendKey + publicViewKey
    checksum = _cn.cn_fast_hash(data)
    return _b58.encode(data + checksum[0:8])

def encode_integrated_addr(version, publicSpendKey, publicViewKey, paymentID):
    '''Given address version, public spend and view keys, and 64-bit payment ID, derive address.'''
    data = version + publicSpendKey + publicViewKey + paymentID
    checksum = _cn.cn_fast_hash(data)
    return _b58.encode(data + checksum[0:8])

def decode_addr(addr):
    '''Given address, get version and public spend and view keys.'''
    d = _b58.decode(addr)
    addr_checksum = d[-8:]
    calc_checksum = _cn.cn_fast_hash(d[:-8])[:8]
    if addr_checksum == calc_checksum:
        version = d[:2]
        publicSpendKey = d[2:66]
        publicViewKey = d[66:130]
        return version, publicSpendKey, publicViewKey
    else:
        return "Invalid Address", [], []

def decode_integrated_addr(addr):
    '''Given address, get version and public spend and view keys.'''
    d = _b58.decode(addr)
    addr_checksum = d[-8:]
    calc_checksum = _cn.cn_fast_hash(d[:-8])[:8]
    if addr_checksum == calc_checksum:
        version = d[:2]
        publicSpendKey = d[2:66]
        publicViewKey = d[66:130]
        paymentID = d[130:146]
        return version, publicSpendKey, publicViewKey, paymentID
    else:
        return "Invalid Address", [], []

def make_integrated_addr(addr, paymentID=None):
    '''Given address and 64-bit payment ID, generate integrated address.'''
    if paymentID == None:
        paymentID = _utils.gen_payment_id('integrated')
    _, psk, pvk = decode_addr(addr)
    vers = '13'
    return encode_integrated_addr(vers, psk, pvk, paymentID)

def account_from_spend_key(sk, acct_type='simplewallet'):
    '''Given a private spend key, derive private view key and address.

    Inputs:
      - sk (str) -- private spend key
      - acct_type (str, optional) -- 'simplewallet' (default) OR 'mymonero'

    Example:
      sk, vk, addr = account_from_seed(seed)

    Outputs:
      - sk (str) -- private spend key
      - vk (str) -- private view key
      - addr (str) -- Monero address
    '''
    if acct_type == 'mymonero':
        sk_hashed = _cn.cn_fast_hash(sk)
        vk = get_view_key(sk_hashed)
        sk = _cn.sc_reduce(sk_hashed)
    elif acct_type == 'simplewallet':
        sk = _cn.sc_reduce(sk)
        vk = get_view_key(sk)
    else:
        raise Exception("Account type not valid.")

    pk = _cn.public_from_secret(sk)
    pvk = _cn.public_from_secret(vk)

    addr = encode_addr(ADDRESS_VERSION, pk, pvk)

    return sk, vk, addr

def account_from_seed(seed, acct_type='simplewallet'):
    '''Given a wallet seed, derive private spend and view keys and address.

    Inputs:
      - seed (list) -- list of seed words from which to derive account info
      - acct_type (str, optional) -- 'simplewallet' (default) OR 'mymonero'

    Example:
      sk, vk, addr = account_from_seed(seed)

    Outputs:
      - sk (str) -- private spend key
      - vk (str) -- private view key
      - addr (str) -- Monero address
    '''
    sk = _mn.mn_decode(seed)
    return account_from_spend_key(sk, acct_type)

def gen_new_wallet_seed():
    '''Generate a new, secure 25-word Monero wallet seed.

    Example:
      seed = gen_new_wallet_seed()

    Outputs:
      - seed (list) -- mnemonic recovery seed
    '''
    sk = _utils.gen_random_hex()
    seed = _mn.mn_encode(sk)
    seed.append(_mn.mn_checksum(seed))
    return seed

def gen_new_wallet():
    '''Generate a secure new wallet, including mnemonic seed, spend and view
    keys, and wallet address.

    Example:
      seed, sk, vk, addr = gen_new_wallet()

    Outputs:
      - seed (list) -- mnemonic recovery seed
      - sk (str) -- private spend key
      - vk (str) -- private view key
      - addr (str) -- Monero address
    '''
    seed = gen_new_wallet_seed()
    sk, vk, addr = account_from_seed(seed)

    return seed, sk, vk, addr

def check_address_viewkey(addr, privateViewKey):
    '''Check that a private viewkey belongs to an address.

    Example:
      isGood = check_address_viewkey(addr, privViewKey)

    Output:
      - isGood (bool)
    '''
    vs, pk, vk = decode_addr(addr)
    viewKeyValid = False
    if vk == _cn.public_from_secret(privateViewKey):
        viewKeyValid = True
    return viewKeyValid



import unittest
from testdata import valid_seeds, valid_sks, invalid_seeds
from context import moneropy

mn = moneropy.mnemonic

class TestMnemonic(unittest.TestCase):

    def test_checksum_valid(self):
        def T(seed):
            self.assertTrue(mn.mn_validate_checksum(seed))

        for i in range(len(valid_seeds)):
            # Input 13 or 25 word seed.
            # Checks that last word matches checksum calculation
            T(valid_seeds[i])

    def test_checksum_invalid(self):
        def T(seed):
            self.assertFalse(mn.mn_validate_checksum(seed))

        for i in range(len(invalid_seeds)):
            T(invalid_seeds[i])

    def test_checksum(self):
        def T(seed, checksum):
            self.assertEqual(mn.mn_checksum(seed), checksum)

        for i in range(len(valid_seeds)):
            # Checksum is the last word in seed.
            T(valid_seeds[i], valid_seeds[i][-1])

            # Can also input seed with or without checksum.
            T(valid_seeds[i][:24], valid_seeds[i][-1])

    def test_mn_decode(self):
        def T(seed, sk):
            self.assertEqual(mn.mn_decode(seed), sk)

        for i in range(len(valid_seeds)):
            T(valid_seeds[i], valid_sks[i])

    def test_mn_encode(self):
        def T(sk, seed):
            self.assertEqual(mn.mn_encode(sk), seed)

        for i in range(len(valid_sks)):
            T(valid_sks[i], valid_seeds[i][:24])


if __name__ == '__main__':
    unittest.main()



import sys as _sys
import os as _os
from binascii import hexlify
from binascii import unhexlify
from .crypto import ed25519 as ed25519
from . import base58 as _b58

PYTHON_VERSION = _sys.version_info.major

if PYTHON_VERSION == 3:
    def hexStrToInt(h):
        '''Converts a hexidecimal string to an integer.'''
        return int.from_bytes(unhexlify(h), "little")

    def intToHexStr(i):
        '''Converts an integer to a hexidecimal string.'''
        return hexlify(i.to_bytes(32, "little")).decode("latin-1")

else:
    def hexStrToInt(h):
        '''Converts a hexidecimal string to an integer.'''
        byt = unhexlify(h)
        return sum((2**i * ed25519.bit(byt, i)) for i in range(0, len(h)*4))

    def intToHexStr(i):
        '''Converts an integer to a hexidecimal string.'''
        return hexlify(ed25519.encodeint(i))

def gen_random_hex(n_bytes=32):
    '''Generate a secure random hexadecimal string.

    By default, this will generate a 32-byte string. However, it can generate a
    string of any byte size by inputing the desired number of bytes.

    Example:
    --------
        h = get_random_hex(8)
    '''
    h = hexlify(_os.urandom(n_bytes))
    return "".join(h.decode("utf-8"))

def gen_payment_id(id_type='normal'):
    '''Generate a 64-bit or 32-byte random hexadecimal payment ID.'''
    if id_type == 'integrated': # 64-bit Payment ID for integrated addresses
        pid = gen_random_hex(8)
    else: # 32-byte regular hexadecimal payment ID
        pid = gen_random_hex(32)
    return pid

def parse_extra_hex(h):
    '''Parse the extra field of a Monero transaction.

    Example:
      pub, pay_id = parse_extra(b)

    Input:
      - h (hex) -- tx extra field

    Outputs:
      - pub (str) -- hexadecimal transaction public key
      - pay_id (str) -- hexadecimal transaction payment id
    '''
    pub = ''
    pay_id = ''
    b = _b58._hexToBin(h)

    if b[0] == 1: # pubkey is tag 1
        pub = h[2:66] # pubkey is 32 bytes # = _b58._binToHex(b[1:33])

        if len(b) >= 66:
            if b[33] == 2 and b[35] == 0 or b[35] == 1:
                pay_id = h[72:(72+b[34]*2-2)] # = _b58._binToHex(b[36:(36+b[34]-1)])

    elif b[0] == 2:
        if b[2] == 0 or b[2] == 1:
            pay_id = h[6:(6+b[1]*2-2)] # = _b58._binToHex(b[3:(3+b[1]+1)]) #

        # second byte of nonce is nonce payload length
        # pubkey tag location: payload length + nonce tag byte + payload length byte
        if b[2+b[1]] == 1:
            offset = (2+b[1]) * 2 # = 2 + b[1]
            pub = h[(offset+2):(offset+2+64)] # = _b58._binToHex(b[(offset+1):(offset+1+32)])

    return pub, pay_id

def parse_extra_bin(b):
    '''Parse the extra field of a Monero transaction.

    Example:
      pub, pay_id = parse_extra(b)

    Input:
      - b (bin) -- tx extra field

    Outputs:
      - pub (str) -- hexadecimal transaction public key
      - pay_id (str) -- hexadecimal transaction payment id
    '''
    pub = ''
    pay_id = ''
    h = _b58._binToHex(b)

    if b[0] == 1: # pubkey is tag 1
        pub = h[2:66] # pubkey is 32 bytes # = _b58._binToHex(b[1:33])

        if len(b) >= 66:
            if b[33] == 2 and b[35] == 0 or b[35] == 1:
                pay_id = h[72:(72+b[34]*2-2)] # = _b58._binToHex(b[36:(36+b[34]-1)])

    elif b[0] == 2:
        if b[2] == 0 or b[2] == 1:
            pay_id = h[6:(6+b[1]*2-2)] # = _b58._binToHex(b[3:(3+b[1]+1)])

        # second byte of nonce is nonce payload length
        # pubkey tag location: payload length + nonce tag byte + payload length byte
        if b[2+b[1]] == 1:
            offset = (2+b[1]) * 2 # = 2 + b[1]
            pub = h[(offset+2):(offset+2+64)] # = _b58._binToHex(b[(offset+1):(offset+1+32)])

    return pub, pay_id



import os
from setuptools import setup, find_packages

# Utility function to read the README file.
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name = "ph4_moneropy",
    version = "0.0.1",
    description = "A python toolbox for Monero.",
    long_description=read('README.md'),
    url = "https://github.com/monero-monitor/moneropy",
    keywords = "monero",
    packages=find_packages(),
    install_requires=requirements,
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Topic :: Utilities",
    ],
    license = "BSD-3-Clause"
)



# MoneroPy - A python toolbox for Monero
# Copyright (C) 2016 The MoneroPy Developers.
#
# MoneroPy is released under the BSD 3-Clause license. Use and redistribution of
# this software is subject to the license terms in the LICENSE file found in the
# top-level directory of this distribution.

from binascii import hexlify, unhexlify
from .crypto import ed25519
from . import utils
#from .crypto.Keccak import Keccak
import sys
import sha3

b = ed25519.b
q = ed25519.q
l = ed25519.l

_hexStrToInt = utils.hexStrToInt
_intToHexStr = utils.intToHexStr

def cn_fast_hash(s):
    return keccak_256(unhexlify(s))

def keccak_256(s):
    #return Keccak().Keccak((len(s)*4, s), 1088, 512, 0x01, 32*8, False).lower()
    k = sha3.keccak_256()
    k.update(s)
    return k.hexdigest()

def sc_reduce(key):
    return _intToHexStr(_hexStrToInt(key) % l)

def sc_reduce32(key):
    return _intToHexStr(_hexStrToInt(key) % q)

def public_from_int(i):
    pubkey = ed25519.encodepoint(ed25519.scalarmultbase(i))
    return hexlify(pubkey)

def public_from_secret(sk):
    return public_from_int(_hexStrToInt(sk)).decode('utf-8')

scalarmult_base = public_from_secret



# The reference Ed25519 software is in the public domain.
#     Source: https://ed25519.cr.yp.to/python/ed25519.py
#     Date accessed: 2 Nov. 2016
#
# Additions at bottom are based on Shen Noether's ed25519.py in MiniNero.
#     Source: https://github.com/monero-project/mininero/blob/master/ed25519.py
#     Most recent commit: bda9d2c33d1baeb4fefccfcc16105b131535cdf1
#     Date accessed: 2 Nov. 2016
#
# Additions and changes made to the original sources are released as specified
# in 'LICENSE' document distributed with this software.

import hashlib
import operator as _oper
import sys as _sys

# Set up byte handling for Python 2 or 3
if _sys.version_info.major == 2:
    int2byte = chr
    range = xrange

    def indexbytes(buf, i):
        return ord(buf[i])

    def intlist2bytes(l):
        return b"".join(chr(c) for c in l)
else:
    indexbytes = _oper.getitem
    intlist2bytes = bytes
    int2byte = _oper.methodcaller("to_bytes", 1, "big")

b = 256
q = 2**255 - 19
l = 2**252 + 27742317777372353535851937790883648493

def H(m):
    return hashlib.sha512(m).digest()

def expmod(b, e, m):
    if e == 0: return 1
    t = expmod(b, e//2, m)**2 % m
    if e & 1: t = (t*b) % m
    return t

def inv(x):
  return expmod(x, q-2, q)

d = -121665 * inv(121666)
I = expmod(2, (q-1)//4, q)

def xrecover(y):
    xx = (y*y-1) * inv(d*y*y+1)
    x = expmod(xx, (q+3)//8, q)
    if (x*x - xx) % q != 0: x = (x*I) % q
    if x % 2 != 0: x = q-x
    return x

By = 4 * inv(5)
Bx = xrecover(By)
B = [Bx%q, By%q]

def edwards(P, Q):
    x1 = P[0]
    y1 = P[1]
    x2 = Q[0]
    y2 = Q[1]
    x3 = (x1*y2+x2*y1) * inv(1+d*x1*x2*y1*y2)
    y3 = (y1*y2+x1*x2) * inv(1-d*x1*x2*y1*y2)
    return [x3%q, y3%q]

def scalarmult(P, e):
    if e == 0: return [0, 1]
    Q = scalarmult(P, e//2)
    Q = edwards(Q, Q)
    if e & 1: Q = edwards(Q, P)
    return Q

def encodeint(y):
    bits = [(y >> i) & 1 for i in range(b)]
    return b''.join([int2byte(sum([bits[i*8 + j] << j for j in range(8)])) for i in range(b//8)])

def encodepoint(P):
    x = P[0]
    y = P[1]
    bits = [(y >> i) & 1 for i in range(b-1)] + [x & 1]
    return b''.join([int2byte(sum([bits[i * 8 + j] << j for j in range(8)])) for i in range(b//8)])

def bit(h, i):
    return (indexbytes(h, i//8) >> (i%8)) & 1

def publickey(sk):
    h = H(sk)
    a = 2**(b-2) + sum(2**i * bit(h, i) for i in range(3, b-2))
    A = scalarmult(B, a)
    return encodepoint(A)

def Hint(m):
    h = H(m)
    return sum(2**i * bit(h, i) for i in range(2*b))

def signature(m, sk, pk):
    h = H(sk)
    a = 2**(b-2) + sum(2**i * bit(h, i) for i in range(3, b-2))
    r = Hint(intlist2bytes([indexbytes(h, j) for j in range(b//8, b//4)]) + m)
    R = scalarmult(B, r)
    S = (r + Hint(encodepoint(R)+pk+m) * a) % l
    return encodepoint(R) + encodeint(S)

def isoncurve(P):
    x = P[0]
    y = P[1]
    return (-x*x + y*y - 1 - d*x*x*y*y) % q == 0

def decodeint(s):
    return sum(2**i * bit(s, i) for i in range(0, b))

def decodepoint(s):
    y = sum(2**i * bit(s, i) for i in range(0, b-1))
    x = xrecover(y)
    if x & 1 != bit(s, b-1): x = q - x
    P = [x, y]
    if not isoncurve(P): raise Exception("decoding point that is not on curve")
    return P

def checkvalid(s, m, pk):
    if len(s) != b//4: raise Exception("signature length is wrong")
    if len(pk) != b//8: raise Exception("public-key length is wrong")
    R = decodepoint(s[0:b//8])
    A = decodepoint(pk)
    S = decodeint(s[b//8:b//4])
    h = Hint(encodepoint(R) + pk + m)
    if scalarmult(B, S) != edwards(R, scalarmult(A, h)):
        raise Exception("signature does not pass verification")

# ----------

# Copyright (c) 2014-2016, The Monero Project
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are
# permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of
#    conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list
#    of conditions and the following disclaimer in the documentation and/or other
#    materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be
#    used to endorse or promote products derived from this software without specific
#    prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
# THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
# THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

def radix255(x):
    x = x % q
    if x + x > q: x -= q
    x = [x, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    bits = [26, 25, 26, 25, 26, 25, 26, 25, 26, 25]
    for i in range(9):
        carry = (x[i] + 2**(bits[i]-1)) // (2**bits[i])
        x[i] -= carry * 2**bits[i]
        x[i + 1] += carry
    result = ",".join(str(xi) for xi in x)
    return result

def theD():
    return d

def computeA():
    return 2 * ((1-d)%q) * inv((1+d)%q) % q

def sqroot(xx):
    x = expmod(xx, (q+3)//8, q)
    if (x*x - xx) % q != 0:
        x = (x*I) % q
    if (x*x - xx) % q != 0:
        print("no square root!")
    return x

def edwards_Minus(P, Q):
    x1 = P[0]
    y1 = P[1]
    x2 = (-Q[0]) % q
    y2 = Q[1]
    x3 = (x1*y2+x2*y1) * inv(1+d*x1*x2*y1*y2)
    y3 = (y1*y2+x1*x2) * inv(1-d*x1*x2*y1*y2)
    return [x3%q, y3%q]

def scalarmultbase(e):
    if e == 0: return [0, 1]
    Q = scalarmult(B, e//2)
    Q = edwards(Q, Q)
    if e & 1: Q = edwards(Q, B)
    return Q

def decodepointcheck(s):
    y = sum(2**i * bit(s, i) for i in range(0, b-1))
    x = xrecover(y)
    if x & 1 != bit(s, b-1): x = q - x
    P = [x, y]
    if not isoncurve(P):
        quit()
        raise Exception("decoding point that is not on curve")
    return P
